package fotograma;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.codec.digest.DigestUtils;

import almonds.ParseException;
import almonds.ParseObject;
import almonds.ParseQuery;

public class Pessoa {
	// Armazenados no banco de dados
	private String id; // endereo no banco de dados
	private String nome;
	private int nSeg;
	
	// Calculados
	private List<Foto> linhaTempo;
	
	// Inicializa o objeto com dados do BD
	private void init(String id, String nome,
			int nSeg, List<Foto> linhaTempo) {
		this.id = id;
		this.nome = nome;
		this.nSeg = nSeg;
		this.linhaTempo = linhaTempo;
	}
	
	private Pessoa(String id, String nome,
			int nSeg, List<Foto> linhaTempo) {
		this.init(id, nome, nSeg, linhaTempo);
	}
	
	public List<Foto> getLinhaTempo() {
		return this.linhaTempo;
	}
	
	// Nova pessoa no BD
	public Pessoa(String nome, String senha) throws ParseException {
		ParseObject pessoa = new ParseObject("Pessoa");
		pessoa.put("nome", nome);
		pessoa.put("senha", DigestUtils.shaHex(Fotograma.SAL + senha));
		pessoa.put("nseg", 0);
		pessoa.save();
		this.init(pessoa.getObjectId(), nome,
				0, new ArrayList<Foto>());
	}
	
	private static List<Foto> linhaTempo(String nome) throws ParseException {
		ParseQuery qry = new ParseQuery("LinhaTempo");
		qry.whereEqualTo("dono", nome);
		qry.addDescendingOrder("createdAt");
		List<ParseObject> fotos = qry.find();
		List<Foto> lt = new ArrayList<Foto>();
		for(ParseObject foto : fotos) {
			lt.add(new Foto(foto.getString("foto")));
		}
		return lt;
	}

	public static Pessoa login(String nome, String senha) throws ParseException {
		ParseQuery busca = new ParseQuery("Pessoa");
		busca.whereEqualTo("nome", nome);
		List<ParseObject> pessoas = busca.find();
		if(pessoas.isEmpty())
			throw new RuntimeException("pessoa no existe");
		ParseObject pessoa = pessoas.get(0);
		if(pessoa.getString("senha").equals(
				DigestUtils.shaHex(Fotograma.SAL + senha))) {
			String id = pessoa.getObjectId();
			List<Foto> linhaTempo =
					Pessoa.linhaTempo(nome);
			return new Pessoa(id, 
					pessoa.getString("nome"), 
					pessoa.getLong("nseg").intValue(),
					linhaTempo);
		} else
			throw new RuntimeException("senha errada");
	}
	
	public void seguir(String nome) throws ParseException {
		ParseObject rel = new ParseObject("Seguidor");
		rel.put("seguidor", this.nome);
		rel.put("seguido", nome);
		rel.save();
		ParseQuery qrySeguido = new ParseQuery("Pessoa");
		qrySeguido.whereEqualTo("nome", nome);
		List<ParseObject> res = qrySeguido.find();
		ParseObject seguido = res.get(0);
		seguido.put("nseg", seguido.getInteger("nseg") + 1);
		seguido.save();
	}
	
	public String toString() {
		String s = nome + " (" + nSeg + ")";
		for(Foto f : this.linhaTempo) {
			s += "\n" + f;
		}
		return s;
	}
}
